<?php

/**
 * Klassen zur Exception-Behandlung und einzelne Exception-Klassen fr {@link DataSet} und 
 * assoziierte Klassen.
 *
 * PHP4 untersttzt Exception-Behandlung nicht als Sprachkonstrukt.
 * Es gibt Mglichkeiten, Exception-Behandlung in PHP4 recht nah an der aus Java und C++ 
 * gewohnten Syntax zu orientieren (vgl. {@link http://www.zend.com/tips/tips.php?id=162&single=1} ).
 *
 * Hier wird eine wesentlich einfachere Variante implementiert. Grund ist, dass PHP5 
 * Exception-Behandlung als Sprachkonstrukt untersttzt. Statt es mglichst vollkommen 
 * in PHP4 zu simulieren, sollte einfach dieser Code zu PHP5 migriert werden. Das ist zum 
 * gegenwrtigen Zeitpunkt (2004-05-26) noch abzuraten, da von PHP5 bisher nur 
 * Entwicklungsversionen existieren.
 *
 * Um eine sptere Migration zu PHP5 einfach zu machen, erben alle einzelnen Exception-Klassen 
 * von einer Basisklasse {@link DataException}, die bei einer Migration gegen die PHP-interne Klasse 
 * Exception auszutauschen ist. Dabei tritt ein Konflikt mit der Klasse Exception des 
 * verwendeten Testing-Frameworks phpUnit auf, der dann zu lsen ist.
 *
 * @package phpDataSet
 * @author Matthias Ansorg <matthias at ansorgs dot de>
 */

/**
 * Basisklasse fr alle konkreten Exceptions
 *
 * Im Wesentlichen nur eine Vorbereitung, um eine Migration auf PHP5 mit der dort PHP-internen 
 * Klasse "Exception" zu vereinfachen.
 *
 * <b>Das Konzept der Exception-Behandlung in PHP4</b>
 *
 * Alle konkreten Exceptions sind Objekte von Klassen, die von "Exception" abgeleitet wurden und 
 * deren Name mit "Exception" endet. Jede Methode, die Exceptions wirft oder Events erzeugt, 
 * verlangt als  Referenzparameter ein Array. An dieses werden alle neu erzeugten Exceptions 
 * angehngt. Die rufende Methode kann dann die geworfenen Exceptions auswerten.
 *
 * Dieses Exception-Konzept wurde fr die Reimplementierung des Microsoft .NET-DataSets in PHP 
 * entwickelt. Um diese Aufgabe zu vereinfachen, wird es auch gleich anstelle des Ereigniskonzepts 
 * eingesetzt. Damit knnen sowohl Ereignisse als auch Exceptions im zurckgegebenen Array enthalten 
 * sein.
 *
 * Alle konkreten Ereignisse sind Objekte von Klassen, die von "Event" abgeleitet wurden. Ihr Name 
 * kann, muss aber nicht mit "Event" enden.
 *
 * Um die doppelte Verwendung des Arrays fr Ereignisse und Exceptions auszudrcken, sollte der 
 * entsprechende Parameter $signalQueue genannt werden.
 *
 * @package phpDataSet
 * @author Matthias Ansorg <matthias at ansorgs dot de>
 */
class DataException {

  /**
   * textuelle Beschreibung dieser Exception
   *
   * get-Methode dieses Attributs: {@link message}
   *
   * @access private
   * @var string
   */
  var $Message;

  /**
   * Konstruktor mit Angabe einer textuellen Beschreibung der Exception und der werfenden Methode
   *
   * @access public
   * @param string $Message textuelle Beschreibung der Exception, die durch das 
   *   zu erstellende Objekt der Klasse Exception reprsentiert werden soll. Diese 
   *   Beschreibung kann spter durch {@link message} ausgelesen werden.
   * @param string $TargetSite Name der Methode, die diese Exception wirft, d.h. 
   *   der Name der eigenen Methode. Dieser Name kann spter durch {@link targetSite} 
   *   ausgelesen werden. Der Name soll aus dem Namen der Klasse gefolgt vom 
   *   Bereichsauflsungsoperator "::" und dem Namen der Methode der Klasse bestehen.
   *   Bei freien Methoden entfallen Klassenname und "::".
   */
  function DataException($Message, $TargetSite) {
    $this->Message = $Message;
  }
  
  /**
   * textuelle Beschreibung dieser Exception (lesen)
   *
   * Ermittelt eine textuelle Beschreibung dieser Exception.
   * get-Methode des Attributs {@link Message}
   *
   * @access public
   * @return string textuelle Beschreibung dieser Exception
   */
   function message() {
     return $this->Message;
   }
  
  /**
   * Name der Anwendung oder des Objektes, das die Fehlerursache dieser Exception ist
   *
   * get/set-Methode dieses Attributs: {@link source}
   *
   * @access private
   * @var string
   */
  var $Source;
  
  /**
   * Name der Anwendung oder des Objektes, das die Fehlerursache dieser Exception ist (lesen, 
   * schreiben).
   *
   * get/set-Methode des Attributs {@link Source}
   *
   * @access public
   * @param string $Source Wert, den das Attribut {@link Source} annehmen soll. Optional. Der 
   *   Parameter wird nur verwendet, wenn die Methode als set-Methode eingesetzt wird.
   * @return string aktueller Wert des Attributs {@link Source}
   */
  function source($Source = null) {
    if (!is_null($Source)) {
      $this->Source = $Source;
    }
    return $this->Source;
  }
  
  /**
   * Name der Methode, die diese Exception wirft
   *
   * get-Methode dieses Attributs: {@link targetSite}
   *
   * @access private
   * @var string
   */
  var $TargetSite;
  
  /**
   * Name der Methode, die diese Exception wirft (lesen)
   *
   * get-Methode des Attributs {@link TargetSite}
   *
   * @access public
   * @return string Name der Methode, die diese Exception wirft
   */
  function targetSite() {
    return $this->TargetSite;
  }
}

/**
 * Ausgelst, wenn ein Methodenaufruf unzulssig ist aus Grnden, die nicht an den bergebenen 
 * Parametern sondern am Zustand des Objektes selbst liegen.
 *
 * @package phpDataSet
 * @author Matthias Ansorg <matthias at ansorgs dot de>
 */
class InvalidOperationException extends DataException {
  /**
   * Konstruktor mit Angabe einer textuellen Beschreibung der Exception und der werfenden Methode
   *
   * @access public
   * @param string $Message textuelle Beschreibung der Exception, die durch das 
   *   erstellte InvalidOperationException-Objekt reprsentiert wird. Diese 
   *   Beschreibung kann spter durch {@link message} ausgelesen werden.
   * @param string $TargetSite Name der Methode, die diese Exception wirft, d.h. 
   *   der Name der eigenen Methode. Dieser Name kann spter durch {@link targetSite} 
   *   ausgelesen werden. Der Name soll aus dem Namen der Klasse gefolgt vom 
   *   Bereichsauflsungsoperator "::" und dem Namen der Methode der Klasse bestehen.
   *   Bei freien Methoden entfallen Klassenname und "::".
   * @see Exception::Exception()
   */
  function InvalidOperationException($Message, $TargetSite) {
    //Oberklassenkonstruktor aufrufen, ohne desen Namen zu kennen
    $par = get_parent_class($this);
    $this->$par($Message, $TargetSite);
  }
}

?>
