<?php

/**
 * Die Klasse {@link DataRow}
 *
 * @package phpDataSet
 * @author Andreas Krling <akraeling@t-online.de>;
 * @since 20.05.04
 */
 
/**
 * Exceptions, die {@link DataRow} werfen kann
 */
require_once("DataRowException.php");

/**
 * Die Klasse {@link DataRow} reprsentiert eine Zeile von Daten in einer {@link DataTable}
 *
 * @package phpDataSet
 * @author Andreas Krling <akraeling@t-online.de>;
 * @since 20.05.2004
 */
class DataRow {

  /**
   * initialisiert eine Instanz eines DataRow-Objektes
   *
   * Der Konstruktor muss die Aufgabe des Konstruktors DataRow(DataRowBuilder builder) aus dem 
   * MS .NET-Framework nachbilden, jedoch ohne die hier nicht implementierte Klasse DataRowBuilder.
   * Daher die abweichende Parameterverwendung gegenber dem MS .NET-Vorbild.
   *
   * Jede DataRow hat einen Verweis 
   * auf genau eine DataTable, deren Schema sie besitzt. Dieses Schema (in Form einer Kollektion 
   * von DataColumns) kann auch nur ber diesen Verweis auf die Tabelle erreicht werden. Der Verweis 
   * auf eine DataTable bedeutet nun nicht, dass jede DataRow in einer DataTable enthalten sein 
   * muss - das kann ber das Attribut {@link RowState} beeinflusst werden.
   *
   * @access private
   * @param DataTable $table Eine Referenz auf die DataTable, deren Schema die DataRow hat.
   * @param mixed[] $values Ein Feld von Werten in Reihenfolge der Spalten, das die Werte fr 
   *   diese {@link DataRow} bereitstellt. Es knnen dieselben Exceptions wie bei {@link itemArray} auftreten.
   *   Optional. Fehlt dieser Parameter, wird eine {@link DataRow} nur mit einem Schema, aber ohne 
   *   Werte erzeugt.
   * @return Eine neue Instanz von {@link DataRow} oder, wenn whrend der Konstruktorausfhrung 
   *   Exceptions auftraten, der <kbd>null</kbd>-Wert.
   */
  function DataRow(&$table, $values=null) {
 
    $this->Table = $table;
    
    if (isset($values)) {
      $this->itemArray($values);
    }
    
    // whrend der Konstruktorausfhrung sind Exceptions aufgetreten:
    if (count($this->signalQ) != 0) {
      $this = null;
    }
  }

  var $signalQ = array();
  
  
  /**
   * Zum Verweisen auf die Tabelle zu der die Row gehrt
   *
   * get/set-Methode dieses Attributs: {@link table}.
   *
   * @access private
   */     
       
  var $Table;
       
  /**
   * Die DataTable, zu der diese Zeile gehrt
   *
   * get-Methode des Attributs {@link Table}.
   *
   * @access public
   */
   
  function table() {
    return $this->Table;
  }

  /**
   * Zum berprfen, ob diese {@link DataRow} Bestandteil einer {@link DataTable} ist
   *
   * Der Wert <kbd>false</kbd> zeigt an, dass die {@link DataRow} unabhngig von  
   * einer {@link DataTable} ist, d.i. nicht zur DataRowCollection irgendeiner {@link DataTable} gehrt. 
   * Das entspricht DataRowState::Detached im Microsoft .NET Framework. Der Wert <kbd>true</kbd> 
   * zeigt an, dass die {@link DataRow} zur DataRowCollection einer {@link DataTable} gehrt.
   * get/set-Methode dieses Attributs: {@link rowState}.
   *
   * @access private       
   * @var boolean
   */
  var $bRowState = false;
         
  /**
   * berprft, ob diese {@link DataRow} Bestandteil einer {@link DataTable} ist
   *
   * get/set-Methode des Attributs {@link bRowState}.
   *
   * @access public
   */
  function rowState($bRowState=null){
    if(is_null($bRowState)) {
      return $this->bRowState;
    }
    else {
      $this->bRowState = $bRowState;
    }
  }
  
  /**
   * Zum Speichern von Werten der Row in einem Array
   *
   * get/set-Methoden zu diesem Attribut: {@link itemArray} und {@link item}. Fr jede Spalte des 
   * Schemas dieser DataRow wird ein Wert gespeichert. Die Struktur dieses Arrays richtet sich 
   * deshalb nach der des Schemas: ein numerisch indiziertes Array (vgl. Dokumentation zu 
   * DataTable::columns()).
   *
   * Das Schema einer DataRow ist das Schema der DataTable, auf die eine DataRow verweist und ist 
   * auch nur so (d.h. ber {@link table}) zu erreichen.
   *
   * @access private
   */
   
  var $ItemArray = array();

  /**
   * Ruft den Inhalt einer angegebenen Spalte ab oder legt diesen fest.
   *
   * get/set-Methode des Attributs {@link ItemArray}.
   *
   * @access public
   * @param mixed $concernedColumnName Die Spalte, deren Inhalt ausgelesen oder gesetzt werden soll. 
   *   Dieser Parameter kann ein String sein (um die {@link DataColumn} nach ihrem Namen 
   *   auszuwhlen), ein Integer sein (um die {@link DataColumn} nach ihrer Position auszuwhlen; 
   *   die erste Spalte ist an Position 0) oder ein {@link DataColumn}-Objekt sein (um die 
   *   Spalte durch Angabe eines gleichen Objektes auszuwhlen).
   * @param mixed $value Der Wert, auf den die Spalte gesetzt werden soll. Optional, nur bei 
   *   Verwendung als set-Methode ntig.
   * @return Der aktuelle Wert der durch den Parameter $column ausgewhlten Spalte. Wurde die Methode 
   *   als set-Methode verwendet, so ist dies der neue Wert, den diese Spalte angenommen hat.
   */
   
  function item($concernedColumnId, $value=null) {
  
    // string-basierten Spaltenbezeichner auf numerischen zurckfhren:
    if (is_string($concernedColumnId)) {
      foreach ($this->Table->columns() as $myKey=>$myColumn) {
        if ($myColumn->ColumnName() == $concernedColumnId) {
          return $this->item($myKey,$value);
        }
      }
    }
      
    // DataColumn-Objekt-basierten Spaltenbezeichner auf numerischen zurckfhren
    if (is_a($concernedColumnId,'DataColumn')) {
      foreach ($this->Table->columns() as $myKey=>$myColumn) {
        if ($myColumn->Equals($concernedColumnId)) {
          return $this->item($myKey,$value);
        }
      }
    }

    // PRE: numerischer Spaltenbezeichner (wir prfen trotzdem ...)
    if (is_int($concernedColumnId)) {
      if (isset($value)) { // Verwendung als set-Methode
        $this->ItemArray[$concernedColumnId] = $value;
      }
      return $this->ItemArray[$concernedColumnId];
    }
  }
  
  /**
   * Ruft alle Werte fr diese Zeile ber ein Array ab oder legt diese fest.
   *
   * get/set-Methode des Attributs {@link ItemArray}. Whrend {@link item} nur den Wert einer 
   * Spalte dieses Attributs ermittelt oder setzt, fhrt diese Methode das stets fr das 
   * gesamte Attribut durch. Das bedeutet, der Parameter $newItemArray wird stets als vollstndiges 
   * Array aufgefasst: fehlende Schlssel/Wert-Paare werden ergnzt, indem fortlaufende numerische 
   * Indizes als Schlssel und <kbd>null</kbd> als Wert verwendet wird.
   *
   * Um in eine {@link DataColumn} ihren {@link DataColumn::DefaultValue} einzutragen, bergebe 
   * man <kbd>null</kbd>. Wenn das Attribut {@link DataColumn::AutoIncrement} <kbd>true</kbd> ist, 
   * bergebe man ebenfalls <kbd>null</kbd> und erhlt so autoinkementierte Werte in dieser Spalte.
   *
   * @access public
   * @param mixed[] $newItemArray Das numerisch indizierte Array von Werten, das das Attribut 
   *   Attribut {@link ItemArray} als Wert annehmen soll. Dieser Parameter ist optional, er wird nur bei 
   *   Verwendung als set-Methode angegeben.
   * @param object[] $signalQueue Warteschlange, in die {@link itemArray} ggf. eine oder mehrere der 
   *   folgenden Exceptions einfgt:
   *   <ul>
   *     <li>{@link ArgumentException}, wenn das bergebene Array $values grer ist als die Anzahl 
   *       der Spalten in der {@link DataTable}, zu der diese {@link DataRow} gehrt.</li>
   *     <li>{@link InvalidCastException}, wenn ein Wert des bergebenen Arrays $values 
   *       inkompatibel mit dem Typ der entsprechenden Spalte dieser {@link DataRow} ist.</li>
   *     <li>{@link ReadOnlyException}, wenn versucht wurde, einen Wert in eine Spalte zu schreiben, 
   *       deren Attribut {@link DataColumn::ReadOnly} <kbd>true</kbd> ist.</li>
   *     <li>{@link NoNullAllowedException}, wenn versucht wurde, SQL-<kbd>NULL</kbd> in eine 
   *       Spalte zu schreiben, deren Attribut {@link DataColumn::AllowDBNull} <kbd>false</kbd> ist.</li>
   *   </ul>
   *   Dieser Parameter ist optional, er wird nur bei Verwendung als set-Methode angegeben.
   * @todo Deklaration und Implementierung der hier zum Argument $signalQueue beschriebenen 
   *   Exception-Klassen.
   */
   
  function itemArray($newItemArray=null) {

    // bei Verwendung als set-Methode:
    if (isset($newItemArray)) {
      if (count($newItemArray) <= count($this->Table->columns())) {
        // Ergnzen des $newItemArray mit am Ende evtl. fehlenden Schlssel/Wert-Paaren:
        for ($i=1; $i<=count($this->Table->columns())-count($newItemArray); ++$i) {
          $newItemArray[] = null;
        }
	
        //ASSERT: count($this->ItemArray) == count($newItemArray)
        $this->ItemArray = $newItemArray;
      }
      else {
        $this->signalQ[] = new InvalidOperationException("DataRow::itemArray: DataRow  " .
        "with has a schema that is no valid extension of the target DataRow.", "DataRow::itemArray");
      }
    }
    return $this->ItemArray;
  }
  
  
  /**
   * Bestimmt, ob diese Instanz von {@link DataRow} gleich einem bergebenen Objekt ist.
   *
   * Zwei Objekte sind gleich, wenn sie Instanzen derselben Klasse sind und denselben Zustand (d.i. 
   * die Belegung ihrer Attribute) haben.
   *
   * @access public
   * @param object $candidate Objekt, fr das bestimmt werden soll ob es gleich dieser Instanz 
   *   von {@link DataRow} ist.
   * @return boolean <kbd>true</kbd>, wenn diese Instanz von {@link  DataRow} gleich dem bergebenen 
   *   Objekt ist, andernfalls <kbd>false</kbd>.
   */
   
  function Equals($candidate) {
    
    $cnt = 0;     
    foreach($candidate->itemArray() as $va) {
      if($va != $this->ItemArray[$cnt]){
        return false;	        
      }
      $cnt++;
    }
    
    if($this->rowState() != $candidate->rowState()) {
      return false;
    }
    
    // Ein Vergleich von DataTables mit == oder != ist nicht mglich aufgrund zirkulrer 
    // Referenzen. Wir beschrnken und hier darauf, den (eindeutig generierten) Namen der 
    // DataTables zu vergleichen. Eine bessere Lsung wre, DataTable::Equals() zu 
    // implementieren.
    $myTable = $this->table();
    $candidateTable = $candidate->table();
    if($myTable->tableName() != $candidateTable->tableName()) {
      return false;
    }
    return true;
  }
  
  // Die folgenden Methoden werden wohl erst nach phpDataSet 1.0 implementiert:
  
  /*
   * Ermittelt die Kind-{@link DataRow DataRows} dieser {@link DataRow} bezglich einer bergebenen 
   * {@link DataRelation}.
   *
   * @access public
   * @param mixed $dataRelation die {@link DataRelation} bezglich der die 
   *   Kind-{@link DataRow DataRows} dieser {@link DataRow} ermittelt werden sollen.  
   *   Die {@link DataRelation} kann als Objekt bergeben oder durch ihren Namen als String reprsentiert 
   *   bergeben werden.
   * @return DataRow[] Die Kind-{@link DataRow DataRows} dieser {@link DataRow} bezglich der 
   *   bergebenen {@link DataRelation}
   * @todo Implementierung. Ist fr eine konsistente Implementierung schon in der ersten Version von 
   *   phpDataSet wnschenswert, weil es hier um das Handling von Instanzen der (bereits 
   *   implementierten!) Klasse DataRelation geht. Jedoch muss die Implementierung nicht zwingend 
   *   in der ersten Version von phpDataSet (d.i. vor Praktikumsende) geschehen.
   */
  /*
  function GetChildRows($dataRelation) {
  }
  
  function GetParentRow($dataRelation) {
  }
  
  function GetParentRows($dataRelation) {
  }
  
  function SetParentRow($dataRow, $dataRelation = null) {
  }
  */
}
?>
