<?php

/**
 * Die Klasse {@link DataSet}
 *
 * @package phpDataSet
 * @author Matthias Ansorg <matthias at ansorgs dot de>
 */
 
/**
 * Klasse zur Angabe von Aktionen bei Schemadifferenzen zwischen {@link DataTable DataTables}
 */ 
require_once("MissingSchemaAction.php");

require_once("DataExceptions.php");
require_once("Events.php");

/**
 * Speicher fr Daten nach dem relationalen Datenmodell; im Hauptspeicher gehalten.
 *
 * @package phpDataSet
 * @author Matthias Ansorg <matthias at ansorgs dot de>
 *
 * @todo <b>ber die Implementierung des Attributs DataSet::Relations.</b><br>
 *   <b>Was ist DataSet::Relations?</b> Ein Attribut, das eine Kollektion aller Relationen dieses 
 *     {@link DataSet DataSets} darstellt. Diese Kollektion enthlt genau die Objekte in Form 
 *     einer Zusammenfassung, die ber {@link DataTable::$ChildRelations} und 
 *     {@link DataTable::$ParentRelations} erreichbar sind. Zusammenfassung bedeutet dabei:
 *     nderungen an diesem Attribut sollen automatisch an den anderen beiden Kollektionen 
 *     nachvollzogen werden, und andersherum. In jedem {@link DataSet} mit konsistentem Zustand
 *     kann also jedes {@link DataRelation}-Objekt in drei Kollektionen erreicht werden:
 *    <ol>
 *      <li>in {@link DataTable::$ChildRelations} der {@link DataTable}, die die Kindrolle in dieser 
 *        {@link DataRelation} hat</li>
 *      <li>in {@link DataTable::$ParentRelations} der {@link DataTable}, die die Elternrolle in 
 *        dieser {@link DataRelation} hat</li>
 *      <li>in {@link Relations} des {@link DataSet DataSets}, zu der die beiden 
 *        {@link DataTable DataTables} gehren.</li>
 *    </ol>
 *   <b>Wann kann implementiert werden?</b> Dieses Attribut bietet bequemen, aber redundanten 
 *     Zugriff auf die {@link DataRelation DataRelations}. Auf eine Implementierung kann also 
 *     verzichtet werden, zumal sie vor Version 1.0 zur Zeit nicht sinnvoll mglich ist. Derzeit 
 *     werden nmlich PHP-Arrays zur Implementierung von Kollektionen verwendet. Die set-Methode 
 *     gibt eine Referenz auf dieses Array zurck, somit ist (direkter) schreibender Zugriff 
 *     mglich. So gibt es aber keine Mglichkeit fr die set-Methode, nach einem solchen 
 *     direkten Schreibzugriff eigenen Code auszufhren und so die nderungen an den Attributen der 
 *     jeweiligen {@link DataTable DataTables} nachzuvollziehen. Dazu msste die Kollektion als 
 *     Klasse implementiert werden, was erst nach Version 1.0 zusammen mit einer einheitlichen 
 *     berarbeitung aller anderen Kollektionen geschehen wird. Alternativ wre es auch mglich, 
 *     das Attribut DataSet::Relations als Array von Referenzen auf die Arrays 
 *     {@link DataTable::$ParentRelations} und {@link DataTable::$ChildRelations} jeder enthaltenen 
 *     {@link DataTable} zu implementieren, so dass sich nderungen bereits durch die verwendeten 
 *     Sprachmittel zumindest in entweder einem ParentRelations- oder einem ChildRelations-Array 
 *     automatisch auswirken. Das macht jedoch die von diesem Attribut erwartete bequemen 
 *     Bedienung zunichte, der Zugriff ist nicht einfacher als direkt auf die Arrays 
 *     der {@link DataTable DataTables}.
 */
class DataSet {

  /**
   * Konstruktor zur Initialisierung jedes neuen DataSets
   * 
   * Fr jeden Aufruf dieses Konstruktor wird eine neu erzeugte Kopie eines {@link DataSet DataSets}
   * initialisiert. Dazu gehrt, sie mit einem eindeutigen Namen zu versehen. Wird kein Name 
   * als Parameter bergeben, so wird automatisch ein eindeutiger Name generiert.
   *
   * @access public
   * @param string $DataSetName ptionaler benutzerdefinierter Name fr dieses {@link DataSet}
   */
  function DataSet($DataSetName = null) {
    static $objCounter = 0;
    // Startwert auswerten und durch "null" gleichzeitig zurcksetzen:
    $presetValue = $this->presetObjCounter(null);
    if (isset($presetValue)) {
      $objCounter = $presetValue;
    }
    
    if (is_null($DataSetName)) {
      $this->dataSetName("DataSet" . $objCounter);
    }
    else {
      $this->dataSetName($DataSetName);
    }
    ++$objCounter;   
  }
  
  /**
   * Klassenweiter Startwert fr einen Zhler, der zur Generierung eindeutiger Namen fr 
   * {@link DataSet}-Objekte im Konstruktor verwendet wird.
   *
   * Der Startwert wird beim nchsten Aufruf von DataSet::DataSet() ausgewertet und 
   * dabei gleichzeitig gelscht, so dass folgende Aufrufe von DataSet::DataSet() 
   * beim aktuellen Zhlerstand und nicht wieder bei diesem Startwert beginnen. Lschen 
   * bedeutet, dass dieser Startwert auf <kbd>null</kbd> gesetzt wird.
   *
   * Zugriff eigentlich friend, erlaubt auch fr DataSetTest. Gedacht ist diese Methode 
   * dazu, in Tests auch statische Variablen direkt setzen zu knnen, um so ihre aktuellen 
   * Werte zu kennen und sinnvolle Annahmen ber zuknftige Werte in den Tests machen zu 
   * knnen.
   *
   * @access private
   * @param mixed $newPresetValue neuer Wert, den der Startwert annehmen soll. Es kann ein 
   *   Integer oder <kbd>null</kbd> bergeben werden. Letzteres bedeutet ein Lschen des 
   *   Startwertes.
   * @return mixed bisheriger Wert des Startwertes. Es kann ein Integer sein oder <kbd>null</kbd>.
   *   Der letzte Fall bedeutet, dass der Startwert nicht gesetzt ist.
   */
  function presetObjCounter($newPresetValue) {
    static $presetValue;
    $tmp = $presetValue;
    $presetValue = $newPresetValue;
    return $tmp;
  }
  
  /**
   * eindeutiger Name dieses DataSets
   *
   * get/set-Methode dieses Attributs: {@link dataSetName}.
   *
   * @access private
   * @var string
   */
  var $DataSetName;
  
  /**
   * eindeutiger Name dieses DataSets (lesen, schreiben)
   *
   * get/set-Methode des Attributs {@link DataSetName}.
   * 
   * @access public
   * @param string $dataSetName Name, den das {@link DataSet} annehmen soll. Optional.
   * @return string aktueller Name dieses {@link DataSet DataSets}. Entspricht dem Parameter 
   *   dieser Methode, wenn ein solcher angegeben wurde, d.h. wenn die Methode als set-Methode 
   *   verwendet wurde.
   */
  function dataSetName($DataSetName = null) {
    if (!is_null($DataSetName)) {
      $this->DataSetName = $DataSetName;
    }
    return $this->DataSetName;
  }
  
  /**
   * Wahrheitswert, der bestimmt, ob die Constraints dieses DataSets durchgesetzt werden sollen
   * 
   * get/set-Methode dieses Attributs: {@link enforceConstraints}. Standardwert: <kbd>true</kbd>.
   *
   * @access private
   * @var bool
   */
  var $EnforceConstraints = true;
  
  /**
   * Wahrheitswert, der bestimmt, ob die Constraints dieses {@link DataSet DataSets} angewandt 
   * werden sollen(lesen, schreiben).
   *
   * get/set-Methode des Attributs {@link EnforceConstraints}. Fr die gelesenen und zu schreibenden 
   * Wahrheitswerte gilt: <kbd>true</kbd>, um Constraints durchzusetzen, <kbd>false</kbd>, um 
   * Constraints nicht durchzusetzen.
   *
   * @access public
   * @param bool $EnforceConstraints Wahrheitswert, den das Attribut {@link EnforceConstraints} 
   *   annehmen soll. Optional.
   * @return bool Wahrheitswert, den das Attribut {@link EnforceConstraints} angenommen hat. Wurde 
   *   ein Parameter bergeben, so ist der Rckgabewert diesem gleich.
   */
  function enforceConstraints($EnforceConstraints = null) {
    if (!is_null($EnforceConstraints) && ($this->EnforceConstraints != $EnforceConstraints)) {
      $this->EnforceConstraints = $EnforceConstraints;
      //gendertes $EnforceConstraints allen Tabellen mitteilen:
      foreach ($this->Tables as $oneOfMyTables) {
        $oneOfMyTables->enforceConstraints($EnforceConstraints);
      }
    }
    return $this->EnforceConstraints;
  }
  
  /**
   * die Kollektion der Tabellen dieses {@link DataSet DataSets}
   *
   * get/set-Methode zu diesem Attribut: {@link tables}
   *
   * @access private
   * @var DataTable[]
   */
  var $Tables;

  /**
   * die Tabellen dieses {@link DataSet DataSets} als Kollektion (lesen, schreiben)
   *
   * get/set-Methode des Attributs {@link Tables}. Schreiben geschieht durch ndern des 
   * Rckgabewertes, er ist eine Referenz. Das Attribut ein assoziatives Array mit den 
   * Namen der Tabellen als Schlsseln. Wie jedes PHP-Array kann der Zugriff auch mit 
   * numerischem Index erfolgen: <kbd>array_values($dataSet->tables())[0]</kbd>.
   *
   * @access public
   * @return DataTable[] Kollektion aller Tabellen, die in diesem DataSet enthalten sind.
   */
  function & tables() {
    return $this->Tables;
  }  
  
  /**
   * Lscht alle Daten dieses {@link DataSet DataSets}
   *
   * Alle Zeilen aller Tabellen werden entfernt. Die Schemata der Tabellen, d.h. 
   * ihre {@link DataColumn}-Objekte, bleiben jedoch erhalten. Ebenso bleiben die 
   * Beziehungen der Tabellen erhalten, d.h. ihre {@link DataRelation}-Objekte.
   *
   * @access public
   */
  function Clear() {
    foreach ($this->Tables as $oneOfMyTables) {
      $oneOfMyTables->Clear();
    }
  }
  
  /**
   * bringt das DataSet in den Zustand zurck, den es hatte nachdem der Konstruktor ausgefhrt wurde 
   *
   * Wie bei {@link Clear} werden alle Daten gelscht. Darber hinaus werden alle nderungen an der 
   * Struktur gelscht; fr {@link DataSet} als Basisklasse bedeutet das: alle Tabellen und alle 
   * Relationen. Seinen Namen behlt das DataSet jedoch, er wurde ja im Konstruktor festgelegt.
   * 
   * Diese Methode ist hauptschlich bei der Verwendung von typisierten DataSets wertvoll. Dabei 
   * reprsetiert jede von {@link DataSet} abgeleitete Klasse einen Typ eines DataSets, mit 
   * spezifischem Schema, spezifischen Beziehungen der Tabellen und Constraints. Diese werden durch 
   * den Konstruktor erzeugt und mssen durch eine berschriebene Version dieser Methode {@link 
   * Reset} wiederhergestellt werden.
   *
   * @access public
   */ 
  function Reset() {
    $this->Clear();
    // hier darf nicht "unset(...)" stehen, weil damit nicht der exakte Zustand nach Konstruktor-
    // aufruf wiederhergestellt wird: nach Konstruktoraufruf haben $this->Tables und 
    // $this->Relations null-Werte, nach unset(...) sind sie gar nicht mehr vorhanden. Das lsst 
    // sich an der String-Reprsentation eines DataSet-Objektes erkennen.
    $this->Tables = null;
  }
   
  /**
   * fhrt dieses DataSet mit anderen Daten ({@link DataRow DataRows}, {@link DataTable DataTable} 
   * oder {@link DataSet DataSet}) und ihren Schemata zusammen.
   *
   * Die Zusammenfhrung luft wie folgt ab:
   * <ol>
   *   <li>Zusammenfhrung der Schemata. blicherweise wird die {@link Merge}-Methode fr Daten mit 
   *     identischen Schemata verwendet, das ist jedoch nicht zwingend. Zulssinge Schemadifferenzen 
   *     sind zustzliche Spalten genderte Primrschlsseleinstellungen in Tabellen.
   *     Diese eventuellen Zustze werden ermittelt und dann die Schemata 
   *     entsprechend dem Argument <kbd>$onMissingSchema</kbd> behandelt.</li>
   *
   *   <li>Zusammenfhrung der Daten. Dabei werden Daten aus Schemaelementen, die in den Quelldaten 
   *     zustzlich vorhanden sind, nur bernommen, wenn <kbd>$onMissingSchema</kbd> den Wert 
   *     {@link MissingSchemaAction::Add()} oder {@link MissingSchemaAction::AddWithKey()} hat. 
   *     Alle anderen Daten werden aus den jeweils gleichnamigen Tabellen der Quelldaten 
   *     bernommen, vorausgesetzt sie haben nicht denselben Primrschlsselwert wie eine in der 
   *     betreffenden Tabelle der Zieldaten existierende {@link DataRow}. Das nmlich wird als Fehler 
   *     gewertet.</li>
   * </ol>
   *
   * @access public
   * @param mixed $secondDataProvider Daten, die mit diesem ersten DataSet zusammengefhrt werden 
   *   sollen. Zulssig sind: {@link DataRow}[] (sofern alle {@link DataRow}-Objekte denselben 
   *   Wert des Attributs {@link DataRow::$Table} haben), {@link DataTable}, {@link DataSet}.
   * @param &object[] $signalQueue Warteschlange fr Events und Exceptions. Ihre Verwendung ist in 
   *   {@link DataExceptions.php} dokumentiert. Alle Events und Exceptions, die diese Methode 
   *   erzeugt, werden hier angehngt. Das sind hier {@link MergeFailed}-Event und 
   *   {@link InvalidOperationException}.
   * @param string $onMissingSchema Angabe, welche Aktion auszufhren ist wenn das Ziel-DataSet 
   *   Schemaelemente vermisst, die die Quelldaten besitzen. Zulssige Werte sind die der 
   *   statischen Variablen der Klasse {@link MissingSchemaAction}. Optionales Argument. Fehlt es, 
   *   entspricht das der Angabe {@link MissingSchemaAction::Ignore}.
   * @see MissingSchemaAction
   * @todo Beschreibe und implementiere die Fehlerbehandlung, wenn in Quell- und Zieldaten eine 
   *   Zeile mit gleichen Primrschlsselwerten vorkommt und 
   *   {@link EnforceConstraints}<kbd>==true</kbd> ist. Wie im Microsoft .NET-DataSet soll das zu 
   *   einem {@link MergeFailed}-Event fhren. Wird nach Version 1.0 implementiert, weil erst 
   *   dann das Constraint-Handling implementiert wird und die von {@link LoadDataRow} erzeugten 
   *   Exceptions, an denen Merge diesen Fehler nur erkennen kann.
   */
  function Merge($secondDataProvider, & $signalQueue, $onMissingSchema = null) {
    //sichere zu: Schemata differieren nur in zulssiger Weise
    //  D.h.: zu jeder Tabelle aus $secondDataProvider gibt es eine gleichnamige Tabelle 
    //  in $this, von deren Schema sie nur durch zustzliche Spalten differiert.
    if (
       is_array($secondDataProvider) ? 
      (!$this->containsBasicSchemaOf($secondDataProvider[0])) :
      (!$this->containsBasicSchemaOf($secondDataProvider))
    ) {
      // Fehlerfall: generiere InvalidOperationException. Aus der Dokumentation des 
      // .NET-DataSets ist kein Verhalten fr diesen Fall zu ersehen, jedoch wird bei ungleichen 
      // Schemata und $onMissingSchema==MissingSchemaAction::Error() ebenfalls 
      // InvalidOperationException erzeugt ( http://msdn.microsoft.com/library/default.asp? 
      // url=/library/en-us/cpref/html/frlrfsystemdatamissingschemaactionclasstopic.asp )
      $signalQueue[] = new InvalidOperationException("DataSet::Merge: The DataProvider to merge " .
        "with has a schema that is no valid extension of the target DataSet.", "DataSet::Merge");
      return;
    }
    
    if ($onMissingSchema == null) {
      $onMissingSchema = MissingSchemaAction::Ignore();
    }
    
    if (is_a($secondDataProvider, 'DataSet')) {
      //Zusammenfhren auf einfachere Flle zurckfhren:
      //fr jede der Tabellen in $secondDataProvider: $this->Merge() mit dieser Tabelle
      foreach ($secondDataProvider->tables() as $oneOfYourTables) {
        $this->Merge($oneOfYourTables, & $signalQueue, $onMissingSchema);
      }
    }
    else {
      if (is_a($secondDataProvider[0], 'DataRow')) {
        //Zusammenfhren auf einfachere Flle zurckfhren:
        //erzeuge eine neue DataTable aus dem DataRow[] und rufe dafr $this->Merge() auf
        $secondDataProviderTableObj = $secondDataProvider[0]->table();
        $tmpDataTable = new DataTable($secondDataProviderTableObj->tableName());
        
        //Verndern von $tmpDataTable->Columns per Referenz! :
        $tmpDataTableColumns =& $tmpDataTable->columns();
        $tmpDataTableColumns = $secondDataProviderTableObj->columns();
        
        //Verndern von $tmpDataTable->PrimaryKey per Referenz! :
        $tmpDataTablePrimaryKey =& $tmpDataTable->primaryKey();
        $tmpDataTablePrimaryKey = $secondDataProviderTableObj->primaryKey();
        
        //Verndern von $tmpDataTable->Rows per Referenz! :
        $tmpDataTableRows =& $tmpDataTable->rows();
        //$tmpDataTableRows = array_merge($tmpDataTableRows, $secondDataProvider);
        $tmpDataTableRows = $secondDataProvider;

        $this->Merge($tmpDataTable, & $signalQueue, $onMissingSchema);
      }
      else {
        if (is_a($secondDataProvider, 'DataTable')) {        
          $myTables =& $this->tables();
          
          $updatingTable =& $secondDataProvider;
          $updateeTable  =& $myTables[$updatingTable->tableName()];

          //mit den zustzlichen Spalten und gendertem Primrschlssel entsprechend 
          //$onMissingSchema verfahren
          $addAction = MissingSchemaAction::Add();
          $addWithKeyAction = MissingSchemaAction::AddWithKey();
          $errorAction = MissingSchemaAction::Error();
          $ignoreAction = MissingSchemaAction::Ignore();
          
          switch ($onMissingSchema) {
            case $addAction:
              $updateeColumns =& $updateeTable->columns();
              $updateeColumns = $updatingTable->columns();
              break;
            case $addWithKeyAction:
              $updateeColumns =& $updateeTable->columns();
              $updateeColumns = $updatingTable->columns();
              $updateePrimaryKey =& $updateeTable->primaryKey();
              $updateePrimaryKey = $updatingTable->primaryKey();
              break;
            case $errorAction:
              //behandle zustzliche Spalten in $updatingTable als Fehler:
              if ( count($updateeTable->columns()) != count($updatingTable->columns()) ) {
                $signalQueue[] = new InvalidOperationException("DataSet::Merge: The DataSet " .
                  "to merge has extra columns and you specified that this shall be treated as " .
                  "an Error (MissingSchemaAction::Error)", "DataSet::Merge");
                return;
              }
              break;
            case $ignoreAction:
              $updatingColumns =& $updatingTable->columns();
              $updatingColumns = $updateeTable->columns();
              break;
          }
          
          //Inhalte zusammenfhren
          //PRE: die beiden Tabellen haben gleiche Schemata, $updateeTable hat die 
          //  gewnschten Primrschlsseleinstellungen
          foreach ($updatingTable->rows() as $updatingRow) {            
            // problematisch: es gibt bisher keine Mglichkeit, Werte nicht mehr existierender 
            // Spalten aus einer DataRow zu entfernen. Ideal wre, wenn die DataRows automatisch 
            // reagieren wrden, wenn sich das Schema ihrer Tabelle ndert, d.h. beim Entfernen 
            // einer DataColumn den entsprechenden Wert im Array lschen wrden.
            // Hier jedoch besteht bei MissingSchemaAction::Ignore() der Fall, dass 
            // $updatingTable zwar schon weniger Spalten hat, ihre DataRows aber noch nicht 
            // weniger Werte im Array. LoadDataRow() fhrt zu einem Fehler, wenn es mit solchen 
            // Arrays mit zu vielen Werten aufgerufen wird.
            // Bisherige Lsung: wir verkrzen das aus der DataRow erzeugte ItemArray auf soviele 
            // Werte wie die DataRow Spalten hat.
            $updateeTable->LoadDataRow(
              array_slice( $updatingRow->itemArray(), 0, count($updatingTable->columns()) )
            );
          }
        }
      }
    }
  }
  
  /**
   * Ermittle, ob das Schema der Tabellen dieses {@link DataSet DataSets} die Basis fr das 
   * Schema aller bergebenen Tabellen enthlt.
   *
   * Ein Schema ist die Kollektion der {@link DataColumn DataColumns} einer Tabelle (d.h. auch, 
   * die Reihenfolge wird bercksichtigt). Ein Schema ist eine zulssige Erweiterung eines anderen 
   * Schemas, wenn es von diesem anderen Schema nur durch zustzliche, am Ende angefgte 
   * Spalten differiert.
   *
   * containsBasicSchemaOf prft nun nach, ob die Schemata aller bergebenen Tabellen solche 
   * zulssigen Erweiterungen der Schemata der Tabellen in diesem {@link DataSet} sind. Die erste 
   * Voraussetzung ist dabei, dass zu jeder bergebenen Tabelle berhaupt eine gleichnamige 
   * Tabelle in diesem {@link DataSet} existiert.
   *
   * @access private
   * @param  object $DataProvider Enthlt ein oder mehrere Tabellen, fr die nachgeprft wird, 
   *   ob sie zulssige Erweiterungen des Schemas dieses {@link DataSet DataSets} sind. Es knnen 
   *   bergeben werden: {@link DataSet}, {@link DataTable} oder {@link DataRow}.
   * @return bool Angabe, ob die Schemata aller bergebenen Tabellen zulssige Erweiterungen der 
   *   Schemata der entsprechenden Tabellen in diesem {@link DataSet} sind (<kbd>true</kbd>) oder 
   *   nicht (<kbd>false</kbd>).
   */
  function containsBasicSchemaOf($DataProvider) {
    
    switch (get_class($DataProvider)) {
      //get_class() ermittelt in PHP4 den Klassennamen benutzerdefinierter Klassen in 
      //Kleinbuchstaben (http://www.php.net/manual/en/function.get-class.php):
      case 'dataset':
        //Annahme: alle Tabellen aus $DataProvider sind zulssige Erweiterungen des Schemas 
        //  der entsprechenden Tabellen von $this
        $areExtensionSchemas = true;
        foreach ($DataProvider->tables() as $DataProviderTable) {
          $areExtensionSchemas = 
            $areExtensionSchemas && $this->containsBasicSchemaOf($DataProviderTable);
        }
        return $areExtensionSchemas;
        break;
      case 'datarow':
        return $this->containsBasicSchemaOf($DataProvider->table());
        break;
      case 'datatable':
        //Annahme: die Tabelle $DataProvider ist eine zulssige Erweiterung einer existierenden, 
        //  gleichnamigen Tabelle aus $this.
        $isExtensionSchema = true;
        
        //Annahme ist widerlegt, wenn keine gleichnamige Tabelle in $this existiert:
        $concernedTableName = $DataProvider->tableName();
        if (!array_key_exists($concernedTableName, $this->tables())) {
          $isExtensionSchema = false;
          return $isExtensionSchema;
        }
        
        //prfe, ob das Schema von $DataProvider mit den Spalten der gleichnamigen Tabelle 
        //  aus $this beginnt. Ein Schema ist hier ein numerisch indiziertes Array von 
        //  DataColumns (vgl. Dokumentation zu DataTable::column()).
        $thisTableCollection = $this->tables();
        $thisTable = $thisTableCollection[$concernedTableName];
        $thisSchema = $thisTable->columns();
        $extensionSchemaCandidateValues = array_values($DataProvider->columns());
        $columnNum = 0;
        foreach ($thisSchema as $thisSchemaColumn) {
          $isExtensionSchema = $isExtensionSchema && 
            $thisSchemaColumn->Equals($extensionSchemaCandidateValues[$columnNum]) ;
          $columnNum++;
        }
        return $isExtensionSchema;
        break;
    }
  
  }
  
  /**
   * Wandelt das aktuelle {@link DataSet DataSet}-Objekt in eine Instanz einer dynamisch erzeugten 
   * Unterklasse von {@link DataSet DataSet} um.
   *
   * Der Name der dynamisch zu erzeugenden Unterklasse kann als Parameter bergeben werden. Auch 
   * knnen Methoden und Eigenschaften, die zur Implementierung der Unterklasse gehren sollen, 
   * per Parameter als PHP-Quelltext bergeben werden.
   *
   * Dies ist eine Hilfsfunktion fr {@link makeTyped}.
   *
   * @access private
   * @param string $subclassName Name der dynamisch zu erzeugenden Unterklasse. <kbd>$this</kbd> ist nach 
   *   Aufruf dieser Methode eine Instanz einer Klasse mit diesem Namen! Der Name darf keine 
   *   schon existierende Klasse bezeichnen, um Konflikte zu vermeiden.
   * @param string $code PHP-Code, der die Implementierung der dynamisch erzeugten Unterklasse 
   *   darstellen soll. Auch berschreiben von Methoden und Attributen ist zulssig.
   * @todo Diese Methode wird auch in den Klassen {@link DataTable} und {@link DataRow} gebraucht, 
   *   um ihre Instanzen in typisierte Pendants umwandeln zu knnen. Dazu sollte diese Methode 
   *   in eine gemeinsame Oberklasse Typable ausgegliedert werden.
   */
  function extend($subclassName, $code) {
    // neue Unterklasse und Objekt davon generieren
    $className = get_class($this);
    eval("class $subclassName extends $className { $code }");
    $new_this = new $subclassName();
    
    // aktuellen Zustand in das neue Objekt bernehmen
    $vars = get_class_vars($className);
    foreach($vars as $key=>$value) {
      $new_this->$key =& $this->$key;
    }
    
    // $this durch das neue Objekt $new_this ersetzen
    $this = $new_this;
  }
  
  /**
   * Wandelt das {@link DataSet DataSet}-Objekt inkl. seinem gesamten Schema in ein 
   * typisiertes {@link DataSet DataSet} um.
   *
   * Das Schema eines {@link DataSet DataSets} ist die Menge seiner {@link DataTable DataTables}, 
   * {@link DataColumn DataColumns} und {@link DataRelations}. Ein typisiertes {@link DataSet} ist 
   * selbst typisiert (d.h. Instanz  einer Unterklasse von {@link DataSet}) und besitzt auch 
   * typisierte {@link DataTable DataTables}. Das sind Unterklassen von {@link DataTable}. 
   * Typisierte {@link DataTable DataTables} besitzen wiederum typisierte {@link DataRow DataRows}, 
   * das sind Unterklassen von {@link DataRow}.
   *
   * Der Klassenname eines typisierten {@link DataSet DataSets} ergibt sich aus dem Namen des 
   * {@link DataSet}-Objektes (siehe {@link dataSetName}), dem DataSet_ vorangestellt wird.
   * 
   * Typisierte {@link DataSet DataSets} werden erstellt, indem man dem Konstruktor als Parameter 
   * eine XSD-Datei (XML Schema Definition) bergibt, die das Schema des {@link DataSet DataSets} 
   * angibt (wobei diese Funktionalitt erst nach Version 1.0 zur Verfgung stehen wird). Entsprechend 
   * ist diese Methode {@link makeTyped} auch nur zur Verwendung im Konstruktor gedacht, daher private.
   *
   * Die Methoden {@link DataSet::makeTyped()} und {@link DataSet::extend()} sind vollstndig 
   * implementiert, jedoch mssen noch nderungen an {@link DataTable} und {@link DataRow} 
   * vorgenommen werden, um eine vollstndige Typisierung zu ermglichen. Die notwendigen 
   * nderungen sind in den @todo-Tags dokumentiert.
   *
   * @access private
   * @todo Wie hier fr den Zugriff auf die Tables-Collection eines {@link DataSet} gezeigt, muss 
   *   in typisierten {@link DataTable DataTables} der Zugriff auf die Collections 
   *   {@link DataTable::$Rows()}, {@link DataTable::$Columns}, {@link DataTable::$ParentRelations}
   *   und {@link DataTable::$ChildRelations} durch eine get/set-Methode pro Element ermglicht 
   *   werden.
   * @todo Auch die Klassen {@link DataTable} und {@link DataRow} bentigen eine Methode 
   *   makeTyped(). Ihre Grundfunktionalitt sollte also in eine Basisklasse Typable 
   *   ausgegliedert werden. makeTyped() der Basisklasse wird dann durch {@link DataSet}, 
   *   {@link DataTable} und {@link DataRow} genutzt und durch eine eigene spezialisierte 
   *   Version berschrieben (Polymorphie).
   */  
  function makeTyped() {

    //Code generieren fr die Implementierung der typisierten DataSet-Klasse
    //  Er erlaubt den Zugriff auf die Tabellen per Attribut, d.i. per get/set-Methode
    $extendingCode = "";
    foreach (array_keys($this->tables()) as $tblName) {
      $extendingCode = $extendingCode .
        "function & $tblName(\$dataTable = null) {
           if (!is_null(\$dataTable)) {
             \$concernedTable =& \$this->tables(\"$tblName\");
             \$concernedTable = \$dataTable;
           }
           return \$this->tables(\"$tblName\");
         }";
    }

    //eigentliche Typisierung dieses DataSets:
    //  Unterklasse generieren und $this durch eine zustandsgleiche Instanz davon austauschen.
    $this->extend("DataSet_".$this->dataSetName(), $extendingCode);
    
    //die DataTables dieses DataSets typisieren:
    //  dabei typisieren sie selbst dann ihre DataRows.    
    foreach (array_keys($this->tables()) as $tblName) {
      //weil foreach() nur auf einer Kopie arbeitet und sonst Zuweisungen unmglich sind:
      $oneOfMyTables =& $this->tables($tblName); 
      //Methode DataTable::makeTyped() muss noch implementiert werden, dann:
      //$oneOfMyTables->makeTyped();
    }
  }
  
}

?>
